@@ -11,16 +11,16 @@ class WorkflowError(Exception):
     """Base class for errors from the xworkflows module."""
 
 
-class InvalidTransitionError(WorkflowError):
-    """Raised when trying to perform a transition not available from current state."""
-
-
 class AbortTransition(WorkflowError):
     """Raised to prevent a transition from proceeding."""
 
 
-class AbortTransitionSilently(WorkflowError):
-    """Raised to (silently) prevent a transition from proceeding."""
+class InvalidTransitionError(AbortTransition):
+    """Raised when trying to perform a transition not available from current state."""
+
+
+class ForbiddenTransition(AbortTransition):
+    """Raised when the 'check' hook of a transition was defined and returned False."""
 
 
 class State(object):
@@ -70,7 +70,10 @@ class StateList(object):
             yield self._states[name]
 
     def __contains__(self, state):
-        return isinstance(state, State) and state.name in self._states and self._states[state.name] == state
+        if isinstance(state, State):
+            return state.name in self._states and self._states[state.name] == state
+        else:  # Expect a state name
+            return state in self._states
 
 
 class Transition(object):
@@ -93,41 +96,6 @@ class Transition(object):
                                    self.name, self.source, self.target)
 
 
-class TransitionDef(object):
-    """A transition definition.
-
-    Attributes:
-        name (str): the name of the transition
-        source (str list): the name of the source states for the transition
-        target (str): the name of the target state for the transition
-    """
-
-    def __init__(self, name, source, target):
-        self.name = name
-        if isinstance(source, str):
-            source = [source]
-        self.source = source
-        self.target = target
-
-    def transition(self, states):
-        """Convert the TransitionDef into a Transition, binding to given states.
-
-        Args:
-            states (StateList): a list of states to use in the resulting
-                Transition
-
-        Returns:
-            Transition: the Transition that was defined.
-        """
-        sources = [states[source] for source in self.source]
-        target = states[self.target]
-        return Transition(self.name, sources, target)
-
-    def __repr__(self):
-        return '%s(%r, %r, %r)' % (self.__class__.__name__,
-                                   self.name, self.source, self.target)
-
-
 class TransitionList(object):
     """Holder for the transitions of a given workflow."""
 
@@ -135,7 +103,8 @@ class TransitionList(object):
         """Create a TransitionList.
 
         Args:
-            transitions (list of TransitionDef): the transitions to include.
+            transitions (list of (name, source, target) tuple): the transitions
+                to include.
         """
         self._transitions = {}
         self._order = []
@@ -183,25 +152,21 @@ class TransitionList(object):
         return '%s(%r)' % (self.__class__.__name__, self._transitions.values())
 
 
-def _setup_states(sdef, prev=()):
+def _setup_states(state_definitions, prev=()):
     """Create a StateList object from a 'states' Workflow attribute."""
-    sts = list(prev)
-    for state in sdef:
-        if isinstance(state, State):
-            st = state
-        elif isinstance(state, str):
-            st = State(state)
-        elif len(state) == 2:
-            name, title = state
-            st = State(name, title)
+    states = list(prev)
+    for state_def in state_definitions:
+        if len(state_def) != 2:
+            raise TypeError("The 'state' attribute of a workflow should be "
+                "a two-tuple of strings; got %r instead." % (state_def,))
         else:
-            raise TypeError("Elements of the 'state' attribute of a "
-                "workflow should be a State object, a string or a pair of "
-                "strings; got %s instead." % (state,))
-        if st in sts:
-            sts.remove(st)
-        sts.append(st)
-    return StateList(sts)
+            name, title = state_def
+            state = State(name, title)
+        if state in states:
+            # Replacing an existing state
+            states.remove(state)
+        states.append(state)
+    return StateList(states)
 
 
 def _setup_transitions(tdef, states, prev=()):
@@ -217,19 +182,16 @@ def _setup_transitions(tdef, states, prev=()):
     """
     trs = list(prev)
     for transition in tdef:
-        if isinstance(transition, TransitionDef):
-            tr = transition.transition(states)
-        elif len(transition) == 3:
+        if len(transition) == 3:
             (name, source, target) = transition
-            # TODO: check that 'source' and 'target' are State list/object.
-            if isinstance(target, State):
-                tr = Transition(name, source, target)
-            else:
-                tr = TransitionDef(name, source, target).transition(states)
+            if isinstance(source, basestring) or isinstance(source, State):
+                source = [source]
+            source = [states[src] for src in source]
+            target = states[target]
+            tr = Transition(name, source, target)
         else:
             raise TypeError("Elements of the 'transition' attribute of a "
-                "workflow should be a TransitionDef object, a string or a "
-                "pair of strings; got %s instead." % (transition,))
+                "workflow should be three-tuples; got %r instead." % (transition,))
 
         at = None
         for i, prev_tr in enumerate(trs):
@@ -263,10 +225,11 @@ class TransitionImplementation(object):
         after (callable): optional callable to call *after* performing the
             transition (once the state has changed).
     """
-    def __init__(self, transition, field_name, workflow, implementation, before=None, after=None):
+    def __init__(self, transition, field_name, workflow, implementation, check=None, before=None, after=None):
         self.transition = transition
         self.field_name = field_name
         self.workflow = workflow
+        self.check = check
         self.before = before
         self.implementation = implementation
         self.after = after
@@ -295,22 +258,23 @@ class TransitionImplementation(object):
                 (self.transition, current_state))
 
     def _pre_transition(self, instance, *args, **kwargs):
+        if self.check is not None:
+            if not self.check(instance):
+                raise ForbiddenTransition(
+                    "Transition %s was forbidden by "
+                    "custom pre-transition check." % self.transition.name)
+
         if self.before is not None:
-            return self.before(instance, *args, **kwargs)
-        return True
+            self.before(instance, *args, **kwargs)
 
     def _run_implem(self, instance, *args, **kwargs):
         """Run the transition, with all checks."""
 
         self._check_state(instance)
         # Call hooks.
-        if not self._pre_transition(instance, *args, **kwargs):
-            return
+        self._pre_transition(instance, *args, **kwargs)
 
-        try:
-            res = self._during_transition(instance, *args, **kwargs)
-        except AbortTransitionSilently:
-            return
+        res = self._during_transition(instance, *args, **kwargs)
 
         from_state = getattr(instance, self.field_name)
         setattr(instance, self.field_name, self.transition.target)
@@ -344,28 +308,31 @@ class TransitionWrapper(object):
         func (function): the decorated method
     """
 
-    def __init__(self, trname, before=None, after=None):
+    def __init__(self, trname, field='', check=None, before=None, after=None):
         self.trname = trname
+        self.field = field
+        self.check = check
         self.before = before
         self.after = after
         self.func = None
 
     def __call__(self, func):
         self.func = func
+        if self.trname == '':
+            self.trname = func.__name__
         return self
 
     def __repr__(self):
         return "<%s for %r: %s>" % (self.__class__.__name__, self.trname, self.func)
 
 
-def transition(trname, before=None, after=None):
-    """Decorator to declare a function as a transition implementation.
-
-    This should be used only when that function should be used for a transition
-    with a different name.
-    """
-
-    return TransitionWrapper(trname, before=before, after=after)
+def transition(trname='', field='', check=None, before=None, after=None):
+    """Decorator to declare a function as a transition implementation."""
+    if callable(trname):
+        raise ValueError("The @transition decorator should be called as "
+            "@transition(['transition_name'], **kwargs)")
+    return TransitionWrapper(trname, field=field,
+        check=check, before=before, after=after)
 
 
 def noop(instance, *args, **kwargs):
@@ -373,13 +340,6 @@ def noop(instance, *args, **kwargs):
     pass
 
 
-class NoOpTransitionImplementation(TransitionImplementation):
-    """A dummy transition implementation which does not perform any action."""
-
-    def __init__(self, transition_name, field_name, workflow):
-        super(NoOpTransitionImplementation, self).__init__(transition_name, field_name, workflow, noop)
-
-
 class ImplementationList(object):
     """Stores all implementations.
 
@@ -398,66 +358,56 @@ class ImplementationList(object):
         self._transitions_mapping = {}
         self._workflow = workflow
 
+    def _get_implem(self, transition_name):
+        return self._implems[self._transitions_mapping[transition_name]]
+
     def collect(self, attrs):
         """Collect the implementations from a given attributes dict."""
-        # Store the transition name => attribute name mapping for
-        # implementations discovered in the attrs dict
-        _local_mappings = {}
 
-        # Store the transition name => callable for callable which may be used
-        # as transition implementations
-        _remaining_candidates = {}
-
-        def add_implem(transition, attr_name, function, before=None, after=None):
+        def add_implem(transition, attr_name, function, check=None, before=None, after=None):
+            """Helper for adding a an implementation to the list."""
             implem = self._workflow.implementation_class(
                 transition, self.state_field, self._workflow, function,
-                before=before, after=after)
+                check=check, before=before, after=after)
             self._implems[attr_name] = implem
-            _local_mappings[transition.name] = attr_name
+            self._transitions_mapping[transition.name] = attr_name
 
-        # First, try to find all TransitionWrapper.
         for name, value in attrs.iteritems():
             if isinstance(value, TransitionWrapper):
                 if value.trname in self._workflow.transitions:
+                    if value.field and value.field != self.state_field:
+                        # Applying for another workflow
+                        continue
+
                     transition = self._workflow.transitions[value.trname]
-                    if value.trname in _local_mappings:
+
+                    if value.trname in self._transitions_mapping:
+                        # Some other function used @transition(value.trname)
+                        other_implem = self._get_implem(value.trname)
                         raise ValueError(
                             "Error for attribute %s: it defines implementation "
                             "%s for transition %s, which is already implemented "
                             "as %s." % (name, value, transition,
-                                self._implems[_local_mappings[value.trname]]))
+                                self._get_implem(value.trname)))
 
-                    add_implem(transition, name, value.func, value.before, value.after)
-
-            elif callable(value):
-                if name in self._workflow.transitions:
-                    _remaining_candidates[name] = value
-
-        # Then, browse the remaining transitions and add callable if needed.
-        _implemented = self._transitions_mapping.copy()
-        _implemented.update(_local_mappings)
-
-        for transition in self._workflow.transitions:
-            trname = transition.name
-            if trname in _remaining_candidates:
-                if trname not in _implemented or _implemented[trname] == trname:
-                    value = _remaining_candidates[transition.name]
-                    add_implem(transition, transition.name, value)
-
-        self._transitions_mapping.update(_local_mappings)
+                    add_implem(transition, name, value.func,
+                        check=value.check, before=value.before, after=value.after)
 
     def _assert_may_override(self, implem, other, attrname):
         if isinstance(other, TransitionImplementation):
+            # Another TransitionImplementation, but not the same transition/workflow
             if other.transition != implem.transition or other.field_name != implem.field_name:
                 raise ValueError(
                     "Can't override transition implementation %s=%s with %s" %
                     (attrname, other, implem))
         elif isinstance(other, TransitionWrapper):
+            # A TransitionWrapper, but not the same Transition
             if other.trname != implem.transition.name:
                 raise ValueError(
                     "Can't override transition implementation %s=%s with %s" %
                     (attrname, other, implem))
         elif other != implem.implementation:
+            # Anything else, and not what this TransitionImplementation wrap.
             raise ValueError(
                 "Can't override attribute %s=%s with %s" %
                 (attrname, other, implem))
@@ -468,26 +418,34 @@ class ImplementationList(object):
 
     def update_attrs(self, attrs):
         for transition in self._workflow.transitions:
-            trname = transition.name
+            # Where to put the implementation
+            attrname = transition.name
+
             if transition.name in self._transitions_mapping:
+                # We had a custom implementation
                 attrname = self._transitions_mapping[transition.name]
                 implem = self._implems[attrname]
                 if implem.transition != transition:
+                    # Should never happen...
                     raise ValueError(
                         "Error for transition %s: implementation has been "
                         "overridden by %s." % (transition, implem))
+            elif attrname in attrs:
+                # No custom implementation, but name conflict.
+                raise ValueError(
+                    "Error for transition %s: no implementation is defined, "
+                    "and the related attribute lacks a @transition "
+                    "decorator: %s" % (transition, attrs[attrname]))
+
             else:
-                attrname = transition.name
-                if attrname in attrs:
-                    raise ValueError(
-                        "Error for transition %s: no implementation is defined, "
-                        "and the related attribute is not callable: %s" %
-                        (transition, attrs[attrname]))
+                # No custom implementation, no name conflict.
+                # Create a 'noop' transition
+                implem = self._workflow.implementation_class(
+                    transition, self.state_field, self._workflow, noop)
 
-                implem = NoOpTransitionImplementation(
-                    transition, self.state_field, self._workflow)
             if attrname in attrs:
                 self._assert_may_override(implem, attrs[attrname], attrname)
+
             self._add_implem(attrs, attrname, implem)
         return attrs
 
@@ -528,9 +486,6 @@ class WorkflowMeta(type):
         return new_class
 
 
-logger = logging.getLogger(__name__)
-
-
 class Workflow(object):
     """Base class for all workflows.
 
@@ -560,6 +515,7 @@ class Workflow(object):
         Kwargs:
             Any passed when calling the transition
         """
+        logger = logging.getLogger('xworkflows.transitions')
         logger.info(u'%r performed transition %s.%s (%s -> %s)', instance,
             self.__class__.__name__, transition.name, from_state.name,
             transition.target.name)
@@ -577,6 +533,8 @@ class StateWrapper(object):
             setattr(self, 'is_%s' % st.name, st.name == self.state.name)
 
     def __eq__(self, other):
+        if isinstance(other, self.__class__):
+            return self.state == other.state
         if isinstance(other, State):
             return self.state == other
         elif isinstance(other, basestring):
