@@ -8,7 +8,7 @@ from xworkflows import base
 
 class WorkflowDeclarationTestCase(unittest2.TestCase):
 
-    def assertExpected(self, workflow, capitalized_states=True, initial_state='foo'):
+    def assertExpected(self, workflow, initial_state='foo'):
         self.assertEqual(3, len(workflow.states))
         self.assertEqual(3, len(workflow.transitions))
         self.assertEqual(workflow.states[initial_state], workflow.initial_state)
@@ -16,43 +16,11 @@ class WorkflowDeclarationTestCase(unittest2.TestCase):
         self.assertEqual(workflow.states['bar'], workflow.transitions['foobar'].target)
 
         for state in workflow.states:
-            if capitalized_states:
-                exp_title = state.name.capitalize()
-            else:
-                exp_title = state.name
+            exp_title = state.name.capitalize()
             self.assertEqual(exp_title, state.title)
             self.assertIn(state.name, ('foo', 'bar', 'baz'))
 
-
     def test_simple_definition(self):
-        class MyWorkflow(base.Workflow):
-            states = ('foo', 'bar', 'baz')
-            transitions = (
-                ('foobar', 'foo', 'bar'),
-                ('gobaz', ('foo', 'bar'), 'baz'),
-                ('bazbar', 'baz', 'bar'),
-            )
-            initial_state = 'foo'
-
-        self.assertExpected(MyWorkflow, capitalized_states=False)
-
-    def test_object_definition(self):
-        class MyWorkflow(base.Workflow):
-            states = (
-                base.State('foo', 'Foo'),
-                base.State('bar', 'Bar'),
-                base.State('baz', 'Baz'),
-            )
-            transitions = (
-                base.TransitionDef('foobar', 'foo', 'bar'),
-                base.TransitionDef('gobaz', ('foo', 'bar'), 'baz'),
-                base.TransitionDef('bazbar', 'baz', 'bar'),
-            )
-            initial_state = 'foo'
-
-        self.assertExpected(MyWorkflow)
-
-    def test_alt_simple_definition(self):
         class MyWorkflow(base.Workflow):
             states = (
                 ('foo', 'Foo'),
@@ -70,7 +38,11 @@ class WorkflowDeclarationTestCase(unittest2.TestCase):
 
     def test_subclassing(self):
         class MyWorkflow(base.Workflow):
-            states = ('foo', 'bar', 'baz')
+            states = (
+                ('foo', "Foo"),
+                ('bar', "Bar"),
+                ('baz', "Baz"),
+            )
             transitions = (
                 ('foobar', 'foo', 'bar'),
                 ('gobaz', ('foo', 'bar'), 'baz'),
@@ -78,12 +50,12 @@ class WorkflowDeclarationTestCase(unittest2.TestCase):
             )
             initial_state = 'foo'
 
-        self.assertExpected(MyWorkflow, capitalized_states=False)
+        self.assertExpected(MyWorkflow)
 
         class MySubWorkflow(MyWorkflow):
             initial_state = 'bar'
 
-        self.assertExpected(MySubWorkflow, capitalized_states=False, initial_state='bar')
+        self.assertExpected(MySubWorkflow, initial_state='bar')
 
     def test_invalid_definitions(self):
         def create_invalid_workflow_1():
@@ -107,7 +79,11 @@ class WorkflowDeclarationTestCase(unittest2.TestCase):
 
         def create_invalid_workflow_3():
             class MyWorkflow(base.Workflow):
-                states = ('foo', 'bar', 'baz')
+                states = (
+                    ('foo', "Foo"),
+                    ('bar', "Bar"),
+                    ('baz', "Baz"),
+                )
                 transitions = (
                     ('foobar', 'bbb', 'bar'),
                 )
@@ -117,7 +93,11 @@ class WorkflowDeclarationTestCase(unittest2.TestCase):
 
         def create_invalid_workflow_4():
             class MyWorkflow(base.Workflow):
-                states = ('foo', 'bar', 'baz')
+                states = (
+                    ('foo', "Foo"),
+                    ('bar', "Bar"),
+                    ('baz', "Baz"),
+                )
                 transitions = (
                     ('foobar', 'bbb'),
                 )
@@ -129,7 +109,11 @@ class WorkflowDeclarationTestCase(unittest2.TestCase):
 class WorkflowEnabledTestCase(unittest2.TestCase):
     def setUp(self):
         class MyWorkflow(base.Workflow):
-            states = ('foo', 'bar', 'baz')
+            states = (
+                ('foo', "Foo"),
+                ('bar', "Bar"),
+                ('baz', "Baz"),
+            )
             transitions = (
                 ('foobar', 'foo', 'bar'),
                 ('gobaz', ('foo', 'bar'), 'baz'),
@@ -173,7 +157,11 @@ class WorkflowEnabledTestCase(unittest2.TestCase):
 
     def test_renamed_implementation(self):
         class MyWorkflow(base.Workflow):
-            states = ('foo', 'bar', 'baz')
+            states = (
+                ('foo', "Foo"),
+                ('bar', "Bar"),
+                ('baz', "Baz"),
+            )
             transitions = (
                 ('foobar', 'foo', 'bar'),
                 ('gobaz', ('foo', 'bar'), 'baz'),
@@ -195,7 +183,11 @@ class WorkflowEnabledTestCase(unittest2.TestCase):
 
     def test_override_renamed(self):
         class MyWorkflow(base.Workflow):
-            states = ('foo', 'bar', 'baz')
+            states = (
+                ('foo', "Foo"),
+                ('bar', "Bar"),
+                ('baz', "Baz"),
+            )
             transitions = (
                 ('foobar', 'foo', 'bar'),
                 ('gobaz', ('foo', 'bar'), 'baz'),
@@ -217,7 +209,11 @@ class WorkflowEnabledTestCase(unittest2.TestCase):
 
     def test_override_conflict(self):
         class MyWorkflow(base.Workflow):
-            states = ('foo', 'bar', 'baz')
+            states = (
+                ('foo', "Foo"),
+                ('bar', "Bar"),
+                ('baz', "Baz"),
+            )
             transitions = (
                 ('foobar', 'foo', 'bar'),
                 ('gobaz', ('foo', 'bar'), 'baz'),
@@ -240,7 +236,11 @@ class WorkflowEnabledTestCase(unittest2.TestCase):
 
     def test_override_with_invalid_wrapper(self):
         class MyWorkflow(base.Workflow):
-            states = ('foo', 'bar', 'baz')
+            states = (
+                ('foo', "Foo"),
+                ('bar', "Bar"),
+                ('baz', "Baz"),
+            )
             transitions = (
                 ('foobar', 'foo', 'bar'),
                 ('gobaz', ('foo', 'bar'), 'baz'),
@@ -263,7 +263,11 @@ class WorkflowEnabledTestCase(unittest2.TestCase):
 
     def test_override_with_constant(self):
         class MyWorkflow(base.Workflow):
-            states = ('foo', 'bar', 'baz')
+            states = (
+                ('foo', "Foo"),
+                ('bar', "Bar"),
+                ('baz', "Baz"),
+            )
             transitions = (
                 ('foobar', 'foo', 'bar'),
                 ('gobaz', ('foo', 'bar'), 'baz'),
@@ -295,7 +299,11 @@ class WorkflowEnabledTestCase(unittest2.TestCase):
 
     def test_dual_workflows(self):
         class MyAltWorkflow(base.Workflow):
-            states = ('foo', 'bar', 'baz')
+            states = (
+                ('foo', "Foo"),
+                ('bar', "Bar"),
+                ('baz', "Baz"),
+            )
             transitions = (
                 ('altfoobar', 'foo', 'bar'),
                 ('altgobaz', ('foo', 'bar'), 'baz'),
@@ -320,7 +328,11 @@ class TransitionRunningTestCase(unittest2.TestCase):
 
     def setUp(self):
         class MyWorkflow(base.Workflow):
-            states = ('foo', 'bar', 'baz')
+            states = (
+                ('foo', "Foo"),
+                ('bar', "Bar"),
+                ('baz', "Baz"),
+            )
             transitions = (
                 ('foobar', 'foo', 'bar'),
                 ('gobaz', ('foo', 'bar'), 'baz'),
@@ -361,7 +373,11 @@ class TransitionRunningTestCase(unittest2.TestCase):
 
     def test_dual_workflows(self):
         class MyAltWorkflow(base.Workflow):
-            states = ('foo', 'bar', 'baz')
+            states = (
+                ('foo', "Foo"),
+                ('bar', "Bar"),
+                ('baz', "Baz"),
+            )
             transitions = (
                 ('altfoobar', 'foo', 'bar'),
                 ('altgobaz', ('foo', 'bar'), 'baz'),
@@ -380,12 +396,99 @@ class TransitionRunningTestCase(unittest2.TestCase):
         self.assertEqual(self.MyWorkflow.states.foo, obj.state1)
         self.assertEqual(MyAltWorkflow.states.foo, obj.state2)
 
+    def test_dual_workflows_conflict(self):
+        """Define an object with two workflows and conflicting transitions."""
+
+        class MyAltWorkflow(base.Workflow):
+            states = (
+                ('foo2', "Foo"),
+                ('bar2', "Bar"),
+                ('baz2', "Baz"),
+            )
+            transitions = (
+                ('foobar', 'foo2', 'bar2'),
+                ('gobaz', ('foo2', 'bar2'), 'baz2'),
+                ('bazbar', 'baz2', 'bar2'),
+            )
+            initial_state = 'foo2'
+
+        def create_invalid_workflow_object():
+            class MyWorkflowObject(base.WorkflowEnabled):
+                state1 = self.MyWorkflow()
+                state2 = MyAltWorkflow()
+
+        self.assertRaises(ValueError, create_invalid_workflow_object)
+
+        def create_other_invalid_workflow_object():
+            """Resolve only some transitions."""
+            class MyWorkflowObject(base.WorkflowEnabled):
+                state1 = self.MyWorkflow()
+                state2 = MyAltWorkflow()
+
+                @base.transition('foobar', field='state2')
+                def foobar2(self):
+                    pass
+
+                @base.transition('gobaz', field='state2')
+                def gobaz2(self):
+                    pass
+
+        self.assertRaises(ValueError, create_other_invalid_workflow_object)
+
+    def test_dual_workflows_conflict_resolved(self):
+        """Define an object with two workflows and renamed transitions."""
+
+        class MyAltWorkflow(base.Workflow):
+            states = (
+                ('foo2', "Foo"),
+                ('bar2', "Bar"),
+                ('baz2', "Baz"),
+            )
+            transitions = (
+                ('foobar', 'foo2', 'bar2'),
+                ('gobaz', ('foo2', 'bar2'), 'baz2'),
+                ('bazbar', 'baz2', 'bar2'),
+            )
+            initial_state = 'foo2'
+
+        class MyWorkflowObject(base.WorkflowEnabled):
+            state1 = self.MyWorkflow()
+            state2 = MyAltWorkflow()
+
+            @base.transition('foobar', field='state2')
+            def foobar2(self):
+                pass
+
+            @base.transition('gobaz', field='state2')
+            def gobaz2(self):
+                pass
+
+            @base.transition('bazbar', field='state2')
+            def bazbar2(self):
+                pass
+
+        obj = MyWorkflowObject()
+        self.assertEqual(self.MyWorkflow.states.foo, obj.state1)
+        self.assertEqual(MyAltWorkflow.states.foo2, obj.state2)
+
+        obj.foobar()
+        self.assertEqual(self.MyWorkflow.states.bar, obj.state1)
+        self.assertEqual(MyAltWorkflow.states.foo2, obj.state2)
+
+        obj.gobaz2()
+        self.assertEqual(self.MyWorkflow.states.bar, obj.state1)
+        self.assertEqual(MyAltWorkflow.states.baz2, obj.state2)
+
 
 class CustomImplementationTestCase(unittest2.TestCase):
 
     def setUp(self):
         class MyWorkflow(base.Workflow):
-            states = ('foo', 'bar', 'baz')
+            states = (
+                ('foo', "Foo"),
+                ('bar', "Bar"),
+                ('baz', "Baz"),
+            )
             transitions = (
                 ('foobar', 'foo', 'bar'),
                 ('gobaz', ('foo', 'bar'), 'baz'),
@@ -412,6 +515,7 @@ class CustomImplementationTestCase(unittest2.TestCase):
         class MyWorkflowObject(base.WorkflowEnabled):
             state = self.MyWorkflow()
 
+            @base.transition()
             def gobaz(self, res):
                 return res + res
 
@@ -425,6 +529,7 @@ class CustomImplementationTestCase(unittest2.TestCase):
         class MyWorkflowObject(base.WorkflowEnabled):
             state = self.MyWorkflow()
 
+            @base.transition()
             def foobar(self, res):
                 return res + res
 
@@ -438,16 +543,21 @@ class CustomImplementationTestCase(unittest2.TestCase):
         class MyWorkflowObject(base.WorkflowEnabled):
             state = self.MyWorkflow()
 
-            def foobar(self):
-                raise base.AbortTransitionSilently()
+            def check_foobar(self):
+                return False
 
+            @base.transition(check=check_foobar)
+            def foobar(self):
+                pass
+
+            @base.transition()
             def gobaz(self):
                 raise KeyError()
 
         obj = MyWorkflowObject()
 
         self.assertEqual(self.MyWorkflow.states.foo, obj.state)
-        self.assertEqual(None, obj.foobar())
+        self.assertRaises(base.ForbiddenTransition, obj.foobar)
         self.assertEqual(self.MyWorkflow.states.foo, obj.state)
 
         self.assertRaises(KeyError, obj.gobaz)
@@ -459,7 +569,11 @@ class ExtendedTransitionImplementationTestCase(unittest2.TestCase):
 
     def setUp(self):
         class MyWorkflow(base.Workflow):
-            states = ('foo', 'bar', 'baz')
+            states = (
+                ('foo', "Foo"),
+                ('bar', "Bar"),
+                ('baz', "Baz"),
+            )
             transitions = (
                 ('foobar', 'foo', 'bar'),
                 ('gobaz', ('foo', 'bar'), 'baz'),
@@ -483,9 +597,11 @@ class ExtendedTransitionImplementationTestCase(unittest2.TestCase):
         class MyWorkflowObject(base.WorkflowEnabled):
             state = MyWorkflow()
 
+            @base.transition()
             def foobar(self, **kwargs):
                 return 1
 
+            @base.transition()
             def gobaz(self, blah=10):
                 return blah
 
